@@ -6,7 +6,6 @@ import (
 	"image/jpeg"
 	"path/filepath"
 	"strconv"
-	"strings"
 	"sync"
 	"sync/atomic"
 	"time"
@@ -15,6 +14,8 @@ import (
 	inv "github.com/darkhz/invidtui/invidious"
 	mp "github.com/darkhz/invidtui/mediaplayer"
 	"github.com/darkhz/invidtui/ui/app"
+	"github.com/darkhz/invidtui/ui/keybinding"
+	"github.com/darkhz/invidtui/ui/theme"
 	"github.com/darkhz/invidtui/utils"
 	"github.com/darkhz/tview"
 	"github.com/gdamore/tcell/v2"
@@ -25,6 +26,7 @@ import (
 type Player struct {
 	queue   Queue
 	fetcher Fetcher
+	seeker  CustomSeeker
 
 	infoID, thumbURI string
 	init             bool
@@ -41,6 +43,8 @@ type Player struct {
 	quality      *tview.DropDown
 	title, desc  *tview.TextView
 
+	property theme.ThemeProperty
+
 	ctx                           context.Context
 	cancel, infoCancel, imgCancel context.CancelFunc
 
@@ -60,47 +64,47 @@ func setup() {
 
 	player.init = true
 
+	player.property = theme.ThemeProperty{
+		Context: theme.ThemeContextPlayer,
+		Item:    theme.ThemeBackground,
+	}
+
 	player.channel = make(chan bool, 10)
 	player.events = make(chan struct{}, 100)
 
-	player.title, player.desc = tview.NewTextView(), tview.NewTextView()
-	player.desc.SetDynamicColors(true)
-	player.title.SetDynamicColors(true)
+	player.title, player.desc =
+		theme.NewTextView(player.property),
+		theme.NewTextView(player.property)
 	player.desc.SetTextAlign(tview.AlignCenter)
 	player.title.SetTextAlign(tview.AlignCenter)
-	player.desc.SetBackgroundColor(tcell.ColorDefault)
-	player.title.SetBackgroundColor(tcell.ColorDefault)
 
-	player.image = tview.NewImage()
-	player.image.SetBackgroundColor(tcell.ColorDefault)
+	player.image = theme.NewImage(
+		player.property.SetContext(theme.ThemeContextPlayerInfo),
+	)
 	player.image.SetDithering(tview.DitheringFloydSteinberg)
 
-	player.info = tview.NewTextView()
-	player.info.SetDynamicColors(true)
+	player.info = theme.NewTextView(
+		player.property.SetContext(theme.ThemeContextPlayerInfo),
+	)
 	player.info.SetTextAlign(tview.AlignCenter)
-	player.info.SetBackgroundColor(tcell.ColorDefault)
 
-	player.quality = tview.NewDropDown()
-	player.quality.SetLabel("[green::b]Quality: ")
-	player.quality.SetBackgroundColor(tcell.ColorDefault)
-	player.quality.SetFieldTextColor(tcell.ColorOrangeRed)
-	player.quality.SetFieldBackgroundColor(tcell.ColorDefault)
-	player.quality.List().
-		SetMainTextColor(tcell.ColorWhite).
-		SetBackgroundColor(tcell.ColorDefault).
-		SetBorder(true)
+	player.quality = theme.NewDropDown(
+		player.property.SetContext(theme.ThemeContextPlayerInfo),
+		"Quality:",
+	)
+	player.quality.List().SetBorder(true)
 
-	player.flex = tview.NewFlex().
+	player.flex = theme.NewFlex(player.property).
 		SetDirection(tview.FlexRow).
 		AddItem(player.title, 1, 0, false).
 		AddItem(player.desc, 1, 0, false)
-	player.flex.SetBackgroundColor(tcell.ColorDefault)
 
-	player.region = tview.NewFlex().
+	player.region = theme.NewFlex(
+		player.property.SetContext(theme.ThemeContextPlayerInfo),
+	).
 		SetDirection(tview.FlexRow).
 		AddItem(player.image, 0, 1, false).
 		AddItem(player.info, 0, 1, false)
-	player.region.SetBackgroundColor(tcell.ColorDefault)
 
 	player.lock = semaphore.NewWeighted(10)
 }
@@ -110,6 +114,7 @@ func Start() {
 	setup()
 	player.queue.Setup()
 	player.fetcher.Setup()
+	player.seeker.Setup()
 
 	loadState()
 	loadHistory()
@@ -150,7 +155,7 @@ func Show() {
 func ToggleInfo(hide ...struct{}) {
 	if hide != nil || player.toggle.Load() {
 		player.toggle.Store(false)
-		player.infoID = ""
+		infoID("")
 
 		infoContext(true, struct{}{})
 
@@ -167,15 +172,16 @@ func ToggleInfo(hide ...struct{}) {
 	if !player.toggle.Load() && player.status.Load() {
 		player.toggle.Store(true)
 
-		box := tview.NewBox()
-		box.SetBackgroundColor(tcell.ColorDefault)
+		property := player.property.SetContext(theme.ThemeContextPlayerInfo)
+		box := theme.NewBox(property)
+		vbox := app.VerticalLine(property.SetItem(theme.ThemeBorder))
 
 		app.UI.Region.Clear().
-			AddItem(player.region, 25, 0, false).
+			AddItem(player.region, 0, 1, false).
 			AddItem(box, 1, 0, false).
-			AddItem(app.VerticalLine(), 1, 0, false).
+			AddItem(vbox, 1, 0, false).
 			AddItem(box, 1, 0, false).
-			AddItem(app.UI.Pages, 0, 1, true)
+			AddItem(app.UI.Pages, 0, 2, true)
 
 		Resize(0, struct{}{})
 
@@ -198,6 +204,7 @@ func Hide() {
 	sendPlayingStatus(false)
 
 	app.UI.QueueUpdateDraw(func() {
+		player.seeker.Hide()
 		ToggleInfo(struct{}{})
 		app.ResizeModal()
 		app.UI.Layout.RemoveItem(player.flex)
@@ -237,7 +244,6 @@ func Resize(width int, force ...struct{}) {
 
 ResizePlayer:
 	sendPlayerEvents()
-	app.UI.Region.ResizeItem(player.region, (width / 4), 0)
 
 	player.width = width
 }
@@ -306,6 +312,23 @@ func IsQueueEmpty() bool {
 	return player.queue.table == nil || player.queue.Count() == 0
 }
 
+// IsQueueAreaFocused returns whether the queue area is focused.
+func IsQueueAreaFocused() bool {
+	pages := player.queue.pages
+	if pages == nil {
+		return false
+	}
+
+	_, item := pages.GetFrontPage()
+
+	return item.HasFocus()
+}
+
+// IsQueueRecommendsFocused returns whether the queue recommendations page is focused.
+func IsQueueRecommendsFocused() bool {
+	return player.queue.recommends != nil && player.queue.recommends.HasFocus()
+}
+
 // IsHistoryInputFocused returns whether the history search bar is focused.
 func IsHistoryInputFocused() bool {
 	return player.history.input != nil && player.history.input.HasFocus()
@@ -315,44 +338,47 @@ func IsHistoryInputFocused() bool {
 func Keybindings(event *tcell.EventKey) *tcell.EventKey {
 	playerKeybindings(event)
 
-	operation := cmd.KeyOperation(event, cmd.KeyContextQueue, cmd.KeyContextFetcher)
+	operation := keybinding.KeyOperation(event, keybinding.KeyContextQueue, keybinding.KeyContextFetcher)
 
 	switch operation {
-	case cmd.KeyPlayerOpenPlaylist:
+	case keybinding.KeyPlayerOpenPlaylist:
 		app.UI.FileBrowser.Show("Open playlist:", openPlaylist)
 
-	case cmd.KeyPlayerHistory:
+	case keybinding.KeyPlayerHistory:
 		showHistory()
 
-	case cmd.KeyPlayerInfo:
+	case keybinding.KeyPlayerInfo:
 		ToggleInfo()
 
-	case cmd.KeyPlayerInfoScrollDown:
+	case keybinding.KeyPlayerInfoScrollDown:
 		player.info.InputHandler()(tcell.NewEventKey(tcell.KeyDown, ' ', tcell.ModNone), nil)
 		return nil
 
-	case cmd.KeyPlayerInfoScrollUp:
+	case keybinding.KeyPlayerInfoScrollUp:
 		player.info.InputHandler()(tcell.NewEventKey(tcell.KeyUp, ' ', tcell.ModNone), nil)
 		return nil
 
-	case cmd.KeyPlayerInfoChangeQuality:
+	case keybinding.KeyPlayerInfoChangeQuality:
 		changeImageQuality()
 
-	case cmd.KeyPlayerQueueAudio, cmd.KeyPlayerQueueVideo, cmd.KeyPlayerPlayAudio, cmd.KeyPlayerPlayVideo:
+	case keybinding.KeyPlayerQueueAudio, keybinding.KeyPlayerQueueVideo, keybinding.KeyPlayerPlayAudio, keybinding.KeyPlayerPlayVideo:
 		playSelected(operation)
 
-	case cmd.KeyFetcher:
+	case keybinding.KeyFetcher:
 		player.fetcher.Show()
 
-	case cmd.KeyQueue:
+	case keybinding.KeyQueue:
 		player.queue.Show()
 
-	case cmd.KeyQueueCancel:
+	case keybinding.KeyQueueCancel:
 		player.queue.Context(true)
 
-	case cmd.KeyAudioURL, cmd.KeyVideoURL:
+	case keybinding.KeyAudioURL, keybinding.KeyVideoURL:
 		playInputURL(event.Rune() == 'b')
 		return nil
+
+	case keybinding.KeyPlayerSeekCustom:
+		player.seeker.Show()
 	}
 
 	return event
@@ -363,39 +389,39 @@ func Keybindings(event *tcell.EventKey) *tcell.EventKey {
 func playerKeybindings(event *tcell.EventKey) {
 	var nokey bool
 
-	switch cmd.KeyOperation(event, cmd.KeyContextPlayer) {
-	case cmd.KeyPlayerStop:
+	switch keybinding.KeyOperation(event, keybinding.KeyContextPlayer) {
+	case keybinding.KeyPlayerStop:
 		player.setting.Store(false)
 		go Hide()
 
-	case cmd.KeyPlayerSeekForward:
+	case keybinding.KeyPlayerSeekForward:
 		mp.Player().SeekForward()
 
-	case cmd.KeyPlayerSeekBackward:
+	case keybinding.KeyPlayerSeekBackward:
 		mp.Player().SeekBackward()
 
-	case cmd.KeyPlayerTogglePlay:
+	case keybinding.KeyPlayerTogglePlay:
 		mp.Player().TogglePaused()
 
-	case cmd.KeyPlayerToggleLoop:
+	case keybinding.KeyPlayerToggleLoop:
 		player.queue.ToggleRepeatMode()
 
-	case cmd.KeyPlayerToggleShuffle:
+	case keybinding.KeyPlayerToggleShuffle:
 		player.queue.ToggleShuffle()
 
-	case cmd.KeyPlayerToggleMute:
+	case keybinding.KeyPlayerToggleMute:
 		mp.Player().ToggleMuted()
 
-	case cmd.KeyPlayerVolumeIncrease:
+	case keybinding.KeyPlayerVolumeIncrease:
 		mp.Player().VolumeIncrease()
 
-	case cmd.KeyPlayerVolumeDecrease:
+	case keybinding.KeyPlayerVolumeDecrease:
 		mp.Player().VolumeDecrease()
 
-	case cmd.KeyPlayerPrev:
+	case keybinding.KeyPlayerPrev:
 		player.queue.Previous(struct{}{})
 
-	case cmd.KeyPlayerNext:
+	case keybinding.KeyPlayerNext:
 		player.queue.Next(struct{}{})
 
 	default:
@@ -409,16 +435,14 @@ func playerKeybindings(event *tcell.EventKey) {
 
 // playSelected determines the media type according
 // to the key pressed, and plays the currently selected entry.
-func playSelected(key cmd.Key) {
-	if player.queue.IsOpen() {
-		kb := cmd.OperationData(key)
-		player.queue.Keybindings(tcell.NewEventKey(kb.Kb.Key, kb.Kb.Rune, kb.Kb.Mod))
-
+func playSelected(key keybinding.Key) {
+	if player.queue.IsQueueShown() {
+		player.queue.Keybindings(keybinding.KeyEvent(key))
 		return
 	}
 
-	audio := key == cmd.KeyPlayerQueueAudio || key == cmd.KeyPlayerPlayAudio
-	current := key == cmd.KeyPlayerPlayAudio || key == cmd.KeyPlayerPlayVideo
+	audio := key == keybinding.KeyPlayerQueueAudio || key == keybinding.KeyPlayerPlayAudio
+	current := key == keybinding.KeyPlayerPlayAudio || key == keybinding.KeyPlayerPlayVideo
 
 	Play(audio, current)
 
@@ -447,15 +471,25 @@ func playInputURL(audio bool) {
 
 // playFromURL plays the given URL.
 func playFromURL(text string, audio bool) {
-	id, mtype, err := utils.GetVPIDFromURL(text)
+	var timestamp *int64
+
+	id, mtype, ts, err := utils.GetVPIDFromURL(text)
 	if err != nil {
 		app.ShowError(err)
 		return
 	}
 
+	if ts != "" {
+		duration := utils.ConvertDurationToSeconds(ts, struct{}{})
+		if duration >= 0 {
+			timestamp = &duration
+		}
+	}
+
 	info := inv.SearchData{
-		Title: text,
-		Type:  mtype,
+		Title:     text,
+		Type:      mtype,
+		Timestamp: timestamp,
 	}
 
 	if mtype == "video" {
@@ -490,16 +524,23 @@ func loadSelected(info inv.SearchData, audio, current bool) {
 // renderPlayer renders the media player within the app.
 func renderPlayer() {
 	var width int
+	var marker string
 
-	app.UI.QueueUpdateDraw(func() {
-		_, _, width, _ = player.desc.GetRect()
-	})
+	app.UI.Lock()
+	_, _, width, _ = player.desc.GetRect()
+	if m := player.queue.marker; m != nil {
+		marker = m.Text
+	}
+	app.UI.Unlock()
 
-	progress, states := updateProgressAndInfo(width - 10)
-	app.UI.QueueUpdateDraw(func() {
-		player.title.SetText("[::b]" + tview.Escape(player.queue.GetTitle()))
-		player.desc.SetText(progress + " " + player.queue.marker.Text)
-	})
+	title, desc, states := updateProgressAndInfo(
+		player.queue.GetTitle(),
+		marker,
+		width-10,
+	)
+	player.title.SetText(title)
+	player.desc.SetText(desc)
+	app.DrawPrimitives(player.flex)
 
 	player.mutex.Lock()
 	player.states = states
@@ -564,24 +605,28 @@ func changeImageQuality(set ...struct{}) {
 		AddItem(player.info, 0, 1, false)
 
 	player.quality.SetInputCapture(func(event *tcell.EventKey) *tcell.EventKey {
-		switch event.Key() {
-		case tcell.KeyEscape:
+		switch keybinding.KeyOperation(event, keybinding.KeyContextCommon) {
+		case keybinding.KeyClose:
 			app.SetPrimaryFocus()
 			player.region.RemoveItem(player.quality)
 		}
 
 		return event
 	})
-	player.quality.SetDrawFunc(func(screen tcell.Screen, x, y, width, height int) (int, int, int, int) {
-		_, _, w, _ := player.quality.List().GetRect()
+	theme.WrapDrawFunc(
+		player.quality,
+		player.property.SetContext(theme.ThemeContextPlayerInfo),
+		func(screen tcell.Screen, x, y, width, height int) (int, int, int, int) {
+			_, _, w, _ := player.quality.List().GetRect()
 
-		dx := ((width / 2) - w) + 2
-		if dx < 0 {
-			dx = 0
-		}
+			dx := ((width / 2) - w) + 2
+			if dx < 0 {
+				dx = 0
+			}
 
-		return dx, y, width, height
-	})
+			return dx, y, width, height
+		},
+	)
 
 	player.quality.SetOptions(options, func(text string, index int) {
 		if index < 0 {
@@ -597,12 +642,12 @@ func changeImageQuality(set ...struct{}) {
 
 		if uri := video.Thumbnails[index].URL; uri != player.thumbURI {
 			player.thumbURI = uri
-			go renderInfoImage(infoContext(true), player.infoID, filepath.Base(uri), struct{}{})
+			go renderInfoImage(infoContext(true), infoID(), filepath.Base(uri), struct{}{})
 		}
 	})
 	player.quality.SetCurrentOption(pos)
 	player.quality.InputHandler()(
-		tcell.NewEventKey(tcell.KeyEnter, ' ', tcell.ModNone),
+		keybinding.KeyEvent(keybinding.KeySelect),
 		func(p tview.Primitive) {
 			app.UI.SetFocus(p)
 		},
@@ -615,24 +660,33 @@ func changeImageQuality(set ...struct{}) {
 
 // renderInfo renders the track information.
 func renderInfo(video inv.VideoData, force ...struct{}) {
-	if force == nil && (video.VideoID == player.infoID || !player.toggle.Load()) {
+	if force == nil && (video.VideoID == infoID() || !player.toggle.Load()) {
 		return
 	}
 
 	infoContext(true, struct{}{})
 
-	app.UI.QueueUpdateDraw(func() {
-		player.info.SetText("[::b]Loading information...")
+	builder := theme.NewTextBuilder(theme.ThemeContextPlayerInfo)
+
+	app.ConditionalDraw(func() bool {
+		builder.Append(theme.ThemeDescription, "info", "Loading information...")
+		player.info.SetText(builder.Get())
+
 		player.image.SetImage(nil)
+
+		return IsInfoShown()
 	})
 
 	if video.Thumbnails == nil {
-		go func(ctx context.Context, pos int, v inv.VideoData) {
+		go func(ctx context.Context, pos int, v inv.VideoData, b theme.ThemeTextBuilder) {
 			v, err := inv.Video(v.VideoID, ctx)
 			if err != nil {
 				if ctx.Err() != context.Canceled {
-					app.UI.QueueUpdateDraw(func() {
-						player.info.SetText("[::b]No information for\n" + v.Title)
+					app.ConditionalDraw(func() bool {
+						b.Format(theme.ThemeDescription, "info", "No information for\n%s", v.Title)
+						player.info.SetText(b.Get())
+
+						return IsInfoShown()
 					})
 				}
 
@@ -641,36 +695,40 @@ func renderInfo(video inv.VideoData, force ...struct{}) {
 
 			player.queue.SetReference(pos, v, struct{}{})
 			renderInfo(v, struct{}{})
-		}(infoContext(false), player.queue.Position(), video)
+		}(infoContext(false), player.queue.Position(), video, builder)
 
 		return
 	}
 
-	app.UI.QueueUpdateDraw(func() {
-		player.infoID = video.VideoID
+	app.ConditionalDraw(func() bool {
+		infoID(video.VideoID)
 		if player.region.GetItemCount() > 2 {
 			player.region.RemoveItemIndex(1)
 		}
 
-		text := "\n"
+		builder.AppendText("\n")
 		if video.Author != "" {
-			text += fmt.Sprintf("[::bu]%s[-:-:-]\n\n", video.Author)
+			builder.Append(theme.ThemeAuthor, "author", tview.Escape(video.Author))
+			builder.AppendText("\n\n")
 		}
 		if video.PublishedText != "" {
-			text += fmt.Sprintf("[lightpink::b]Uploaded %s[-:-:-]\n", video.PublishedText)
+			builder.Append(theme.ThemePublished, "published", "Uploaded "+video.PublishedText)
+			builder.AppendText("\n\n")
 		}
-		text += fmt.Sprintf(
-			"[aqua::b]%s views[-:-:-] / [red::b]%s likes[-:-:-] / [purple::b]%s subscribers[-:-:-]\n\n",
-			utils.FormatNumber(video.ViewCount),
-			utils.FormatNumber(video.LikeCount),
-			video.SubCountText,
-		)
-		text += "[::b]" + tview.Escape(video.Description)
 
-		player.info.SetText(text)
+		builder.Format(theme.ThemeViews, "views", " %s views / ", utils.FormatNumber(video.ViewCount))
+		builder.Format(theme.ThemeLikes, "likes", "%s likes / ", utils.FormatNumber(video.LikeCount))
+		builder.Format(theme.ThemeSubscribers, "info", "%s subscribers", video.SubCountText)
+		builder.AppendText("\n\n")
+
+		builder.Append(theme.ThemeDescription, "description", tview.Escape(video.Description))
+
+		player.info.SetText(builder.Get())
 		player.info.ScrollToBeginning()
 
 		changeImageQuality(struct{}{})
+
+		return IsInfoShown()
 	})
 
 	go renderInfoImage(infoContext(true), video.VideoID, filepath.Base(player.thumbURI))
@@ -701,8 +759,10 @@ func renderInfoImage(ctx context.Context, id, image string, change ...struct{})
 		return
 	}
 
-	app.UI.QueueUpdateDraw(func() {
+	app.ConditionalDraw(func() bool {
 		player.image.SetImage(thumbnail)
+
+		return IsInfoShown()
 	})
 
 	app.ShowInfo("Player: Image loaded", false, change != nil)
@@ -720,27 +780,37 @@ func playingStatusCheck() {
 		}
 
 		Context(false)
-		go playerUpdateLoop(player.ctx, player.cancel)
+		go playerUpdateLoop(player.ctx)
 	}
 }
 
 // playerUpdateLoop updates the player.
-func playerUpdateLoop(ctx context.Context, cancel context.CancelFunc) {
-	t := time.NewTicker(1 * time.Second)
-	defer t.Stop()
+func playerUpdateLoop(ctx context.Context) {
+	evCtx, evCancel := context.WithCancel(context.Background())
+	go func(c context.Context) {
+		for {
+			select {
+			case <-c.Done():
+				player.desc.Clear()
+				player.title.Clear()
+
+				app.DrawPrimitives(player.flex, app.UI.Status.Pages)
+
+				return
+
+			case <-player.events:
+				renderPlayer()
+			}
+		}
+	}(evCtx)
 
 	for {
 		select {
 		case <-ctx.Done():
-			player.desc.Clear()
-			player.title.Clear()
+			evCancel()
 			return
 
-		case <-player.events:
-			renderPlayer()
-			continue
-
-		case <-t.C:
+		case <-time.After(1 * time.Second):
 			renderPlayer()
 		}
 	}
@@ -751,6 +821,10 @@ func mediaEventHandler(event mp.MediaEvent) {
 	switch event {
 	case mp.EventInProgress:
 		player.queue.MarkPlayingEntry(EntryPlaying)
+		player.queue.SetTimestamp(player.queue.Position())
+
+	case mp.EventLoading:
+		player.queue.MarkPlayingEntry(EntryLoading)
 
 	case mp.EventEnd:
 		player.queue.MarkPlayingEntry(EntryStopped)
@@ -789,8 +863,7 @@ func openPlaylist(file string) {
 // of the currently playing track, and updates the track information.
 //
 //gocyclo:ignore
-func updateProgressAndInfo(width int) (string, []string) {
-	var lhs, rhs string
+func updateProgressAndInfo(title, marker string, width int) (string, string, []string) {
 	var states []string
 
 	eof := mp.Player().Finished()
@@ -802,14 +875,8 @@ func updateProgressAndInfo(width int) (string, []string) {
 
 	duration := mp.Player().Duration()
 	timepos := mp.Player().Position()
-	currtime := utils.FormatDuration(timepos)
 
-	vol := strconv.Itoa(volume)
-	if vol == "" {
-		vol = "0"
-	}
-	states = append(states, "volume "+vol)
-	vol += "%"
+	builder := theme.NewTextBuilder(theme.ThemeContextPlayer)
 
 	if timepos < 0 {
 		timepos = 0
@@ -821,9 +888,6 @@ func updateProgressAndInfo(width int) (string, []string) {
 		timepos = duration
 	}
 
-	totaltime := utils.FormatDuration(duration)
-	mtype := "(" + player.queue.GetMediaType() + ")"
-
 	width /= 2
 	length := width * int(timepos)
 	if duration > 0 {
@@ -835,56 +899,80 @@ func updateProgressAndInfo(width int) (string, []string) {
 		endlength = width
 	}
 
-	if shuffle {
-		lhs += " S"
-		states = append(states, "shuffle")
-	}
-	if mute {
-		lhs += " M"
-		states = append(states, "mute")
-	}
-
-	loop, loopsetting := "", ""
+	loopsetting := ""
 	switch player.queue.GetRepeatMode() {
-	case mp.RepeatModeOff:
-		loop = ""
-
 	case mp.RepeatModeFile:
-		loop = "R-F"
+		builder.Append(theme.ThemeLoop, "loop", "R-F ")
 		loopsetting = "loop-file"
 
 	case mp.RepeatModePlaylist:
-		loop = "R-P"
+		builder.Append(theme.ThemeLoop, "loop", "R-P ")
 		loopsetting = "loop-playlist"
 	}
-	if loop != "" {
+	if loopsetting != "" {
 		states = append(states, loopsetting)
 	}
 
-	state := ""
+	if shuffle {
+		builder.Append(theme.ThemeShuffle, "shuffle", "S ")
+		states = append(states, "shuffle")
+	}
+	if mute {
+		builder.Append(theme.ThemeMute, "mute", "M ")
+		states = append(states, "mute")
+	}
+
 	switch {
 	case paused:
 		if eof {
-			state = "[]"
+			builder.Append(theme.ThemeStop, "stop", "[] ")
 		} else {
-			state = "||"
+			builder.Append(theme.ThemePause, "pause", "|| ")
 		}
 
 	case buffering:
-		state = "B"
+		builder.Append(theme.ThemeBuffer, "buffer", "B")
 		if pct := mp.Player().BufferPercentage(); pct >= 0 {
-			state += "(" + strconv.Itoa(pct) + "%)"
+			builder.Format(theme.ThemeBuffer, "buffer", "(%s%%)", strconv.Itoa(pct))
 		}
+		builder.AppendText(" ")
 
 	default:
-		state = ">"
+		builder.Append(theme.ThemePlay, "play", "> ")
 	}
 
-	rhs = " " + vol + " " + mtype
-	lhs = loop + lhs + " " + state + " "
-	progress := currtime + " |" + strings.Repeat("█", length) + strings.Repeat(" ", endlength) + "| " + totaltime
+	builder.Append(theme.ThemeDuration, "duration", utils.FormatDuration(timepos))
 
-	return (lhs + progress + rhs), states
+	builder.Start(theme.ThemeProgressBar, "progress")
+	builder.AppendText(" |")
+	for i := 0; i < length; i++ {
+		builder.AppendText("█")
+	}
+	for i := 0; i < endlength; i++ {
+		builder.AppendText(" ")
+	}
+	builder.AppendText("| ")
+	builder.Finish()
+
+	builder.Append(theme.ThemeTotalDuration, "totalduration", utils.FormatDuration(duration))
+
+	vol := strconv.Itoa(volume)
+	if vol == "" {
+		vol = "0"
+	}
+	states = append(states, "volume "+vol)
+	builder.Format(theme.ThemeVolume, "volume", " %s%% ", vol)
+	builder.Format(theme.ThemeMediaType, "mediatype", "(%s) ", player.queue.GetMediaType())
+	builder.AppendText(marker)
+
+	title = theme.SetTextStyle(
+		"title",
+		title,
+		player.property.Context,
+		theme.ThemeTitle,
+	)
+
+	return title, builder.Get(), states
 }
 
 // sendPlayingStatus sends status events to the player.
@@ -908,8 +996,23 @@ func sendPlayerEvents() {
 	}
 }
 
+// infoID returns or sets the video ID for the current player information.
+func infoID(set ...string) string {
+	player.mutex.Lock()
+	defer player.mutex.Unlock()
+
+	if set != nil {
+		player.infoID = set[0]
+	}
+
+	return player.infoID
+}
+
 // infoContext returns a new context for loading the player information.
 func infoContext(image bool, all ...struct{}) context.Context {
+	player.mutex.Lock()
+	defer player.mutex.Unlock()
+
 	ctx, cancel := context.WithCancel(context.Background())
 
 	if image {
