@@ -12,10 +12,12 @@ import (
 	"sync"
 	"sync/atomic"
 
-	"github.com/darkhz/invidtui/cmd"
 	inv "github.com/darkhz/invidtui/invidious"
 	mp "github.com/darkhz/invidtui/mediaplayer"
 	"github.com/darkhz/invidtui/ui/app"
+	"github.com/darkhz/invidtui/ui/keybinding"
+	"github.com/darkhz/invidtui/ui/theme"
+	"github.com/darkhz/invidtui/ui/view"
 	"github.com/darkhz/invidtui/utils"
 	"github.com/darkhz/tview"
 	"github.com/etherlabsio/go-m3u8/m3u8"
@@ -28,13 +30,14 @@ import (
 type Queue struct {
 	init, moveMode bool
 	prevrow        int
-	videos         map[string]*inv.VideoData
 
 	status chan struct{}
 
-	modal  *app.Modal
-	table  *tview.Table
-	marker *tview.TableCell
+	modal             *app.Modal
+	table, recommends *tview.Table
+	tabs              *tview.TextView
+	pages             *tview.Pages
+	marker            *tview.TableCell
 
 	lock *semaphore.Weighted
 
@@ -55,10 +58,11 @@ type Queue struct {
 
 // QueueData describes the queue entry data.
 type QueueData struct {
-	URI                       []string
+	URI                       [2]string
 	Reference                 inv.VideoData
 	Columns                   [QueueColumnSize]*tview.TableCell
 	Audio, Playing, HasPlayed bool
+	Timestamp                 *int64
 }
 
 // QueueEntryStatus describes the status of a queue entry.
@@ -76,9 +80,6 @@ const (
 
 	QueuePlayingMarker = QueueColumnSize - 2
 	QueueMediaMarker   = QueueColumnSize - 5
-
-	PlayerMarkerFormat = `[%s::b][%s[][-:-:-]`
-	MediaMarkerFormat  = `[pink::b]%s[-:-:-]`
 )
 
 // Setup sets up the queue.
@@ -87,22 +88,41 @@ func (q *Queue) Setup() {
 		return
 	}
 
+	property := q.ThemeProperty()
+	defer q.tabsHandler()
+
 	q.store = deque.New[*QueueData](100)
 
 	q.status = make(chan struct{}, 100)
-	q.videos = make(map[string]*inv.VideoData)
 
-	q.table = tview.NewTable()
+	q.pages = theme.NewPages(property)
+
+	q.tabs = theme.NewTextView(property)
+	q.tabs.SetTextAlign(tview.AlignCenter)
+
+	q.table = theme.NewTable(property)
 	q.table.SetContent(q)
 	q.table.SetSelectable(true, false)
 	q.table.SetInputCapture(q.Keybindings)
-	q.table.SetBackgroundColor(tcell.ColorDefault)
 	q.table.SetSelectionChangedFunc(q.selectorHandler)
 	q.table.SetFocusFunc(func() {
-		app.SetContextMenu(cmd.KeyContextQueue, q.table)
+		app.SetContextMenu(keybinding.KeyContextQueue, q.table)
 	})
 
-	q.modal = app.NewModal("queue", "Queue", q.table, 40, 0)
+	q.recommends = theme.NewTable(property)
+	q.recommends.SetSelectable(true, false)
+	q.recommends.SetInputCapture(q.Keybindings)
+	q.recommends.SetFocusFunc(func() {
+		app.SetContextMenu(keybinding.KeyContextQueue, q.recommends)
+	})
+
+	flex := theme.NewFlex(property).
+		SetDirection(tview.FlexRow).
+		AddItem(q.pages, 0, 1, true).
+		AddItem(app.HorizontalLine(property), 1, 0, false).
+		AddItem(q.tabs, 1, 0, false)
+
+	q.modal = app.NewModal("queue", "Queue", flex, 40, 0, property)
 
 	q.lock = semaphore.NewWeighted(1)
 
@@ -129,8 +149,15 @@ func (q *Queue) IsOpen() bool {
 	return q.modal != nil && q.modal.Open
 }
 
+// IsQueueShown returns whether the queue page is shown.
+func (q *Queue) IsQueueShown() bool {
+	page, _ := q.pages.GetFrontPage()
+
+	return q.IsOpen() && page == "queue"
+}
+
 // Add adds an entry to the player queue.
-func (q *Queue) Add(video inv.VideoData, audio bool, uri ...string) {
+func (q *Queue) Add(video inv.VideoData, audio bool, uri ...[2]string) {
 	count := q.Count()
 	_, _, w, _ := q.GetRect()
 
@@ -144,62 +171,169 @@ func (q *Queue) Add(video inv.VideoData, audio bool, uri ...string) {
 		length = utils.FormatDuration(video.LengthSeconds)
 	}
 
+	uris := [2]string{}
+	if uri != nil {
+		uris = uri[0]
+	}
+
+	timestamp := video.Timestamp
 	video.MediaType = media
 
 	q.SetData(count, QueueData{
 		Columns: [QueueColumnSize]*tview.TableCell{
-			tview.NewTableCell(" ").
+			theme.NewTableCell(
+				theme.ThemeContextQueue,
+				theme.ThemePopupBackground,
+				" ",
+			).
 				SetMaxWidth(1).
-				SetSelectable(false),
-			tview.NewTableCell("[blue::b]" + tview.Escape(video.Title)).
+				SetSelectable(false).
+				SetReference(AttachableReference(video)),
+			theme.NewTableCell(
+				theme.ThemeContextQueue,
+				theme.ThemeVideo,
+				tview.Escape(video.Title),
+			).
+				SetExpansion(1).
+				SetMaxWidth(w / 7).
+				SetSelectable(true),
+			theme.NewTableCell(
+				theme.ThemeContextQueue,
+				theme.ThemePopupBackground,
+				" ",
+			).
+				SetMaxWidth(1).
+				SetSelectable(true),
+			theme.NewTableCell(
+				theme.ThemeContextQueue,
+				theme.ThemeAuthor,
+				tview.Escape(video.Author),
+			).
 				SetExpansion(1).
 				SetMaxWidth(w / 7).
 				SetSelectable(true).
-				SetSelectedStyle(app.UI.ColumnStyle),
-			tview.NewTableCell(" ").
+				SetAlign(tview.AlignRight),
+			theme.NewTableCell(
+				theme.ThemeContextQueue,
+				theme.ThemePopupBackground,
+				" ",
+			).
 				SetMaxWidth(1).
-				SetSelectable(false),
-			tview.NewTableCell("[purple::b]" + tview.Escape(video.Author)).
-				SetExpansion(1).
-				SetMaxWidth(w / 7).
-				SetSelectable(true).
-				SetAlign(tview.AlignRight).
-				SetSelectedStyle(app.UI.ColumnStyle),
-			tview.NewTableCell(" ").
-				SetMaxWidth(1).
-				SetSelectable(false),
-			tview.NewTableCell(fmt.Sprintf(MediaMarkerFormat, media)).
+				SetSelectable(true),
+			theme.NewTableCell(
+				theme.ThemeContextQueue,
+				theme.ThemeMediaType,
+				media,
+			).
 				SetMaxWidth(5).
-				SetSelectable(true).
-				SetSelectedStyle(app.UI.ColumnStyle),
-			tview.NewTableCell(" ").
+				SetSelectable(true),
+			theme.NewTableCell(
+				theme.ThemeContextQueue,
+				theme.ThemePopupBackground,
+				" ",
+			).
 				SetMaxWidth(1).
-				SetSelectable(false),
-			tview.NewTableCell("[pink::b]" + length).
+				SetSelectable(true),
+			theme.NewTableCell(
+				theme.ThemeContextQueue,
+				theme.ThemeTotalDuration,
+				length,
+			).
 				SetMaxWidth(10).
-				SetSelectable(true).
-				SetSelectedStyle(app.UI.ColumnStyle),
-			tview.NewTableCell(" ").
-				SetMaxWidth(11).
-				SetSelectable(false),
-			tview.NewTableCell(" ").
+				SetSelectable(true),
+			theme.NewTableCell(
+				theme.ThemeContextQueue,
+				theme.ThemePopupBackground,
+				" ",
+			).
+				SetMaxWidth(15).
+				SetSelectable(true),
+			theme.NewTableCell(
+				theme.ThemeContextQueue,
+				theme.ThemePopupBackground,
+				" ",
+			).
 				SetMaxWidth(1).
-				SetSelectable(false),
+				SetSelectable(true),
 		},
 		Audio:     audio,
 		Reference: video,
-		URI:       uri,
+		URI:       uris,
+		Timestamp: timestamp,
 	})
 
 	if count == 0 {
 		q.SwitchToPosition(count)
 
-		app.UI.QueueUpdateDraw(func() {
+		app.ConditionalDraw(func() bool {
 			q.SelectCurrentRow()
+
+			return q.IsOpen()
 		})
 	}
 }
 
+// AddRecommendations adds the video-based recommendations to the queue modal.
+func (q *Queue) AddRecommendations(video inv.VideoData) {
+	q.recommends.Clear()
+	_, _, w, _ := q.GetRect()
+
+	for i, v := range video.RecommendedVideos {
+		length := "Live"
+		if !v.LiveNow {
+			length = utils.FormatDuration(v.LengthSeconds)
+		}
+
+		q.recommends.SetCell(i, 0, theme.NewTableCell(
+			theme.ThemeContextQueue,
+			theme.ThemeVideo,
+			tview.Escape(v.Title),
+		).
+			SetExpansion(1).
+			SetMaxWidth(w/7).
+			SetSelectable(true).
+			SetReference(AttachableReference(v)),
+		)
+		q.recommends.SetCell(i, 1, theme.NewTableCell(
+			theme.ThemeContextQueue,
+			theme.ThemePopupBackground,
+			" ",
+		).
+			SetMaxWidth(1).
+			SetSelectable(true),
+		)
+		q.recommends.SetCell(i, 2, theme.NewTableCell(
+			theme.ThemeContextQueue,
+			theme.ThemeAuthor,
+			tview.Escape(v.Author),
+		).
+			SetExpansion(1).
+			SetMaxWidth(w/7).
+			SetSelectable(true).
+			SetAlign(tview.AlignRight),
+		)
+		q.recommends.SetCell(i, 3, theme.NewTableCell(
+			theme.ThemeContextQueue,
+			theme.ThemePopupBackground,
+			" ",
+		).
+			SetMaxWidth(1).
+			SetSelectable(true),
+		)
+		q.recommends.SetCell(i, 4, theme.NewTableCell(
+			theme.ThemeContextQueue,
+			theme.ThemeTotalDuration,
+			length,
+		).
+			SetMaxWidth(10).
+			SetSelectable(true),
+		)
+	}
+
+	q.recommends.Select(0, 0)
+	q.recommends.ScrollToBeginning()
+}
+
 // AutoPlay automatically selects what to play after
 // the current entry has finished playing.
 func (q *Queue) AutoPlay(force bool) {
@@ -209,7 +343,8 @@ func (q *Queue) AutoPlay(force bool) {
 
 	case mp.RepeatModePlaylist:
 		if !q.shuffle.Load() && q.Position() == q.Count()-1 {
-			q.SwitchToPosition(0)
+			q.SwitchToPosition(0, struct{}{})
+			q.MarkPlayingEntry(EntryPlaying)
 			return
 		}
 	}
@@ -237,7 +372,7 @@ func (q *Queue) Play(norender ...struct{}) {
 
 		q.MarkPlayingEntry(EntryFetching)
 		q.audio.Store(data.Audio)
-		q.title.Store(data.Reference.Title)
+		q.title.Store(tview.Escape(data.Reference.Title))
 
 		sendPlayerEvents()
 		Show()
@@ -253,12 +388,13 @@ func (q *Queue) Play(norender ...struct{}) {
 		}
 
 		q.SetReference(q.Position(), video, struct{}{})
+		q.AddRecommendations(video)
 		q.MarkPlayingEntry(EntryLoading)
 
 		if err := mp.Player().LoadFile(
 			data.Reference.Title, data.Reference.LengthSeconds,
 			data.Audio,
-			uri...,
+			uri,
 		); err != nil {
 			app.ShowError(err)
 			return
@@ -320,7 +456,7 @@ func (q *Queue) SetPosition(position int) {
 }
 
 // SwitchToPosition switches to the specified position within the queue.
-func (q *Queue) SwitchToPosition(position int) {
+func (q *Queue) SwitchToPosition(position int, autoplay ...struct{}) {
 	q.storeMutex.Lock()
 	defer q.storeMutex.Unlock()
 
@@ -328,6 +464,8 @@ func (q *Queue) SwitchToPosition(position int) {
 	if !ok {
 		return
 	}
+
+	skipPlaying := autoplay != nil && position == 0 && data == q.current
 	if q.current != nil {
 		q.current.Playing = false
 	}
@@ -337,6 +475,10 @@ func (q *Queue) SwitchToPosition(position int) {
 
 	q.current = data
 
+	if skipPlaying {
+		return
+	}
+
 	q.SetPosition(position)
 	q.Play()
 }
@@ -531,9 +673,9 @@ func (q *Queue) SelectCurrentRow(row ...int) {
 func (q *Queue) GetRect() (int, int, int, int) {
 	var x, y, w, h int
 
-	app.UI.QueueUpdate(func() {
-		x, y, w, h = q.table.GetRect()
-	})
+	app.UI.RLock()
+	x, y, w, h = q.table.GetRect()
+	app.UI.RUnlock()
 
 	return x, y, w, h
 }
@@ -550,39 +692,40 @@ func (q *Queue) MarkPlayingEntry(status QueueEntryStatus) {
 		return
 	}
 
-	app.UI.QueueUpdateDraw(func() {
+	tagMap := map[QueueEntryStatus]theme.ThemeItem{
+		EntryFetching: theme.ThemeTagFetching,
+		EntryLoading:  theme.ThemeTagLoading,
+		EntryPlaying:  theme.ThemeTagPlaying,
+		EntryStopped:  theme.ThemeTagStopped,
+	}
+
+	app.ConditionalDraw(func() bool {
 		if q.marker != nil {
 			q.marker.SetText("")
 		}
 
+		builder := theme.NewTextBuilder(theme.ThemeContextQueue)
+		builder.Format(tagMap[status], "tag", " %s ", status)
+
 		q.marker = cell
+		q.marker.SetText(builder.Get())
 
-		color := "white"
-		switch status {
-		case EntryFetching, EntryLoading:
-			color = "yellow"
-
-		case EntryStopped:
-			color = "red"
-		}
-
-		marker := string(status)
-		q.marker.SetText(fmt.Sprintf(PlayerMarkerFormat, color, marker))
+		return q.IsOpen()
 	})
 }
 
 // MarkEntryMediaType marks the selected queue entry as 'Audio/Video'.
-func (q *Queue) MarkEntryMediaType(key cmd.Key) {
+func (q *Queue) MarkEntryMediaType(key keybinding.Key) {
 	var media string
 
 	q.storeMutex.Lock()
 	defer q.storeMutex.Unlock()
 
 	switch key {
-	case cmd.KeyPlayerQueueAudio:
+	case keybinding.KeyPlayerQueueAudio:
 		media = "Audio"
 
-	case cmd.KeyPlayerQueueVideo:
+	case keybinding.KeyPlayerQueueVideo:
 		media = "Video"
 
 	default:
@@ -597,9 +740,13 @@ func (q *Queue) MarkEntryMediaType(key cmd.Key) {
 		return
 	}
 
+	position := mp.Player().Position()
+
 	data.Audio = audio
+	data.Timestamp = &position
 	data.Columns[QueueMediaMarker].SetText(
-		fmt.Sprintf(MediaMarkerFormat, media),
+		theme.SetTextStyle(
+			"mediatype", media, theme.ThemeContextQueue, theme.ThemeMediaType),
 	)
 
 	if pos == q.Position() {
@@ -632,6 +779,9 @@ func (q *Queue) SetReference(row int, video inv.VideoData, checkID ...struct{})
 	}
 
 	data.Reference = video
+	if len(data.Columns) > 0 && data.Columns[0] != nil {
+		data.Columns[0].SetReference(AttachableReference(video))
+	}
 }
 
 // SetState sets the player states (repeat/shuffle).
@@ -648,6 +798,25 @@ func (q *Queue) SetState(state string) {
 	}
 }
 
+// SetTimestamp seeks to the available timestamp during playback.
+func (q *Queue) SetTimestamp(position int) {
+	q.storeMutex.Lock()
+	defer q.storeMutex.Unlock()
+
+	data, ok := q.GetEntryPointer(position)
+	if !ok || data.Timestamp == nil {
+		return
+	}
+
+	timestamp := *data.Timestamp
+	if timestamp < 0 {
+		return
+	}
+
+	mp.Player().SetPosition(timestamp)
+	data.Timestamp = nil
+}
+
 // Clear clears the queue.
 func (q *Queue) Clear() {
 	q.storeMutex.Lock()
@@ -834,7 +1003,7 @@ ReadPlaylist:
 			video.HlsURL = mediaURI
 		}
 
-		q.Add(video, audio, mediaURI)
+		q.Add(video, audio, [2]string{mediaURI})
 
 		filesAdded++
 	}
@@ -844,11 +1013,11 @@ ReadPlaylist:
 
 // Keybindings define the keybindings for the queue.
 func (q *Queue) Keybindings(event *tcell.EventKey) *tcell.EventKey {
-	operation := cmd.KeyOperation(event, cmd.KeyContextQueue)
+	operation := keybinding.KeyOperation(event, keybinding.KeyContextQueue, keybinding.KeyContextComments)
 
-	for _, op := range []cmd.Key{
-		cmd.KeyClose,
-		cmd.KeyQueueSave,
+	for _, op := range []keybinding.Key{
+		keybinding.KeyClose,
+		keybinding.KeyQueueSave,
 	} {
 		if operation == op {
 			q.Hide()
@@ -857,31 +1026,37 @@ func (q *Queue) Keybindings(event *tcell.EventKey) *tcell.EventKey {
 	}
 
 	switch operation {
-	case cmd.KeyQueuePlayMove:
+	case keybinding.KeyQueuePlayMove:
 		q.play()
 
-	case cmd.KeyQueueSave:
+	case keybinding.KeyQueueSave:
 		app.UI.FileBrowser.Show("Save as:", q.saveAs)
 
-	case cmd.KeyQueueAppend:
+	case keybinding.KeyQueueAppend:
 		app.UI.FileBrowser.Show("Append from:", q.appendFrom)
 
-	case cmd.KeyQueueDelete:
+	case keybinding.KeyQueueDelete:
 		q.remove()
 
-	case cmd.KeyQueueMove:
+	case keybinding.KeyQueueMove:
 		q.move()
 
-	case cmd.KeyPlayerQueueAudio, cmd.KeyPlayerQueueVideo:
+	case keybinding.KeyPlayerQueueAudio, keybinding.KeyPlayerQueueVideo:
 		q.MarkEntryMediaType(operation)
 
-	case cmd.KeyPlayerStop, cmd.KeyClose:
+	case keybinding.KeyPlayerStop, keybinding.KeyClose:
 		q.Hide()
+
+	case keybinding.KeyComments:
+		view.Comments.Show()
+
+	case keybinding.KeySwitch:
+		q.tabsHandler()
 	}
 
-	for _, o := range []cmd.Key{
-		cmd.KeyQueueMove,
-		cmd.KeyQueueDelete,
+	for _, o := range []keybinding.Key{
+		keybinding.KeyQueueMove,
+		keybinding.KeyQueueDelete,
 	} {
 		if operation == o {
 			app.ResizeModal()
@@ -892,6 +1067,14 @@ func (q *Queue) Keybindings(event *tcell.EventKey) *tcell.EventKey {
 	return event
 }
 
+// ThemeProperty returns the queue's theme property.
+func (q *Queue) ThemeProperty() theme.ThemeProperty {
+	return theme.ThemeProperty{
+		Context: theme.ThemeContextQueue,
+		Item:    theme.ThemePopupBackground,
+	}
+}
+
 // play handles the 'Enter' key event within the queue.
 // If the move mode is enabled, the currently moving item
 // is set to the position where the selector rests.
@@ -965,9 +1148,11 @@ func (q *Queue) move() {
 func (q *Queue) selectorHandler(row, col int) {
 	selector := ">"
 	rows := q.table.GetRowCount()
+	item := theme.ThemeNormalModeSelector
 
 	if q.moveMode {
 		selector = "M"
+		item = theme.ThemeMoveModeSelector
 	}
 
 	for i := 0; i < rows; i++ {
@@ -978,7 +1163,14 @@ func (q *Queue) selectorHandler(row, col int) {
 		}
 
 		if i == row {
-			cell.SetText(selector)
+			cell.SetText(
+				theme.SetTextStyle(
+					"selector",
+					selector,
+					theme.ThemeContextQueue,
+					item,
+				),
+			)
 			continue
 		}
 
@@ -986,6 +1178,61 @@ func (q *Queue) selectorHandler(row, col int) {
 	}
 }
 
+// tabsHandler handles tab switching within the queue.
+func (q *Queue) tabsHandler() {
+	tab := app.Tab{
+		Title: "Queue",
+		Info: []app.TabInfo{
+			{ID: "queue", Title: "Queue"},
+			{ID: "recommends", Title: "Recommendations"},
+		},
+
+		Context: theme.ThemeContextQueue,
+
+		TabView: q.tabs,
+	}
+
+	if q.tabs.GetText(false) == "" {
+		builder := theme.NewTextBuilder(theme.ThemeContextQueue)
+		items := map[string]tview.Primitive{
+			"queue":      q.table,
+			"recommends": q.recommends,
+		}
+
+		for i, info := range tab.Info {
+			q.pages.AddPage(info.ID, items[info.ID], true, info.ID == "queue")
+
+			builder.Format(theme.ThemeTabs, info.ID, " %s ", info.Title)
+			if i < len(tab.Info) {
+				builder.AppendText(" ")
+			}
+		}
+
+		q.tabs.SetText(builder.Get())
+		q.pages.SwitchToPage("recommends")
+	}
+
+	page, _ := q.pages.GetFrontPage()
+	tab.Selected = page
+
+	id := app.SwitchTab(false, tab)
+	q.pages.SwitchToPage(id)
+	for _, info := range tab.Info {
+		if id != info.ID {
+			continue
+		}
+
+		q.modal.Title.SetText(
+			theme.SetTextStyle(
+				"title", info.Title,
+				theme.ThemeContextQueue, theme.ThemeTitle,
+			),
+		)
+
+		break
+	}
+}
+
 // saveAs saves the current queue into a playlist M3U8 file.
 func (q *Queue) saveAs(file string) {
 	if !q.lock.TryAcquire(1) {
@@ -1042,3 +1289,14 @@ func (q *Queue) sendStatus() {
 	default:
 	}
 }
+
+// AttachableReference returns an attachable reference to the video item.
+func AttachableReference(v inv.VideoData) inv.SearchData {
+	return inv.SearchData{
+		Type:     "video",
+		Title:    v.Title,
+		VideoID:  v.VideoID,
+		AuthorID: v.AuthorID,
+		Author:   v.Author,
+	}
+}
